Index: lib/libc/sys/kqueue.2
===================================================================
--- lib/libc/sys/kqueue.2	(revision 264413)
+++ lib/libc/sys/kqueue.2	(working copy)
@@ -24,7 +24,7 @@
 .\"
 .\" $FreeBSD$
 .\"
-.Dd April 7, 2014
+.Dd April 13, 2014
 .Dt KQUEUE 2
 .Os
 .Sh NAME
@@ -465,9 +465,26 @@
 which is controlled by the
 .Va kern.kq_calloutmax
 sysctl.
+.Bl -tag -width XXNOTE_USECONDS
+.It Dv NOTE_SECONDS
+.Va data
+is in seconds.
+.It Dv NOTE_MSECONDS
+.Va data
+is in milliseconds.
+.It Dv NOTE_USECONDS
+.Va data
+is in microseconds.
+.It Dv NOTE_NSECONDS
+.Va data
+is in nanoseconds.
+.It
+.El
 .Pp
-On return,
+If
 .Va fflags
+is not set, the default is milliseconds. On return,
+.Va fflags
 contains the events which triggered the filter.
 .It Dv EVFILT_USER
 Establishes a user event identified by
Index: sys/kern/kern_event.c
===================================================================
--- sys/kern/kern_event.c	(revision 264413)
+++ sys/kern/kern_event.c	(working copy)
@@ -524,14 +524,24 @@
  * interval timer support code.
  */
 static __inline sbintime_t
-timer2sbintime(intptr_t data)
+timer2sbintime(intptr_t data, int flags)
 {
+	sbintime_t modifier;
 
+	if (flags & NOTE_SECONDS)
+		modifier = SBT_1S;
+	else if (flags & NOTE_USECONDS)
+		modifier = SBT_1US;
+	else if (flags & NOTE_NSECONDS)
+		modifier = SBT_1NS;
+	else
+		modifier = SBT_1MS;
+
 #ifdef __LP64__
-	if (data > SBT_MAX / SBT_1MS)
+	if (data > SBT_MAX / modifier)
 		return (SBT_MAX);
 #endif
-	return (SBT_1MS * data);
+	return (modifier * data);
 }
 
 static void
@@ -547,13 +557,13 @@
 	if ((kn->kn_flags & EV_ONESHOT) != EV_ONESHOT) {
 		calloutp = (struct callout *)kn->kn_hook;
 		callout_reset_sbt_on(calloutp,
-		    timer2sbintime(kn->kn_sdata), 0 /* 1ms? */,
+		    timer2sbintime(kn->kn_sdata, kn->kn_sfflags), 0
 		    filt_timerexpire, kn, PCPU_GET(cpuid), 0);
 	}
 }
 
 /*
- * data contains amount of time to sleep, in milliseconds
+ * data contains amount of time to sleep
  */
 static int
 filt_timerattach(struct knote *kn)
@@ -566,7 +576,7 @@
 		return (EINVAL);
 	if ((intptr_t)kn->kn_sdata == 0 && (kn->kn_flags & EV_ONESHOT) == 0)
 		kn->kn_sdata = 1;
-	to = timer2sbintime(kn->kn_sdata);
+	to = timer2sbintime(kn->kn_sdata, kn->kn_sfflags);
 	if (to < 0)
 		return (EINVAL);
 
@@ -583,7 +593,7 @@
 	calloutp = malloc(sizeof(*calloutp), M_KQUEUE, M_WAITOK);
 	callout_init(calloutp, CALLOUT_MPSAFE);
 	kn->kn_hook = calloutp;
-	callout_reset_sbt_on(calloutp, to, 0 /* 1ms? */,
+	callout_reset_sbt_on(calloutp, to, 0
 	    filt_timerexpire, kn, PCPU_GET(cpuid), 0);
 
 	return (0);
Index: sys/sys/event.h
===================================================================
--- sys/sys/event.h	(revision 264413)
+++ sys/sys/event.h	(working copy)
@@ -133,6 +133,12 @@
 #define	NOTE_TRACKERR	0x00000002		/* could not track child */
 #define	NOTE_CHILD	0x00000004		/* am a child process */
 
+/* additional flags for EVFILE_TIMER */
+#define NOTE_SECONDS	0x00000001		/* data is seconds */
+#define NOTE_MSECONDS	0x00000002		/* data is milliseconds */
+#define NOTE_USECONDS	0x00000004		/* data is microseconds */
+#define NOTE_NSECONDS	0x00000008		/* data is nanoseconds */
+
 struct knote;
 SLIST_HEAD(klist, knote);
 struct kqueue;
